#ifndef ADDTOGASHI_H
#define ADDTOGASHI_H
#include "mbed.h"
typedef enum {
    /* PRESETCTRL0 resets */
    RESET_FLASH = 7,        /*!< FLASH controller reset control */
    RESET_EEPROM = 9,       /*!< EEPROM controller reset control */
    RESET_MUX = 11,         /*!< Input mux reset control */
    RESET_IOCON = 13,       /*!< IOCON reset control */
    RESET_PININT = 18,      /*!< Pin interrupt (PINT) reset reset control */
    RESET_GINT,             /*!< Grouped interrupt (GINT) reset control */
    RESET_DMA,              /*!< DMA reset control */
    RESET_CRC,              /*!< CRC reset control */
    RESET_ADC0 = 27,        /*!< ADC0 reset control */
    RESET_ADC1,             /*!< ADC1 reset control */
    RESET_ACMP = 30,        /*!< Analog Comparator (all 4 ACMP) reset control */
    RESET_MRT = 32 + 0,     /*!< Multi-rate timer (MRT) reset control */
    RESET_RIT,              /*!< Repetitive interrupt timer (RIT) reset control */
    RESET_SCT0,             /*!< State configurable timer 0 (SCT0) reset control */
    RESET_SCT1,             /*!< State configurable timer 1 (SCT1) reset control */
    RESET_SCT2,             /*!< State configurable timer 2 (SCT2) reset control */
    RESET_SCT3,             /*!< State configurable timer 3 (SCT3) reset control */
    RESET_SCTIPU,           /*!< State configurable timer IPU (SCTIPU) reset control */
    RESET_CAN,              /*!< CAN reset control */
    RESET_SPI0 = 32 + 9,    /*!< SPI0 reset control */
    RESET_SPI1,             /*!< SPI1 reset control */
    RESET_I2C0 = 32 + 13,   /*!< I2C0 reset control */
    RESET_UART0 = 32 + 17,  /*!< UART0 reset control */
    RESET_UART1,            /*!< UART1 reset control */
    RESET_UART2,            /*!< UART2 reset control */
    RESET_QEI0 = 32 + 21,   /*!< QEI0 reset control */
    RESET_USB = 32 + 23     /*!< USB reset control */
} CHIP_SYSCTL_PERIPH_RESET_T;

/**
 * System and peripheral clocks
 */
typedef enum CHIP_SYSCTL_CLOCK {
    /* Peripheral clock enables for SYSAHBCLKCTRL0 */
    SYSCTL_CLOCK_SYS = 0,               /*!< System clock */
    SYSCTL_CLOCK_ROM,                   /*!< ROM clock */
    SYSCTL_CLOCK_SRAM1 = 3,             /*!< SRAM1 clock */
    SYSCTL_CLOCK_SRAM2,                 /*!< SRAM2 clock */
    SYSCTL_CLOCK_FLASH = 7,             /*!< FLASH controller clock */
    SYSCTL_CLOCK_EEPROM = 9,            /*!< EEPROM controller clock */
    SYSCTL_CLOCK_MUX = 11,              /*!< Input mux clock */
    SYSCTL_CLOCK_SWM,                   /*!< Switch matrix clock */
    SYSCTL_CLOCK_IOCON,                 /*!< IOCON clock */
    SYSCTL_CLOCK_GPIO0,                 /*!< GPIO0 clock */
    SYSCTL_CLOCK_GPIO1,                 /*!< GPIO1 clock */
    SYSCTL_CLOCK_GPIO2,                 /*!< GPIO2 clock */
    SYSCTL_CLOCK_PININT = 18,           /*!< PININT clock */
    SYSCTL_CLOCK_GINT,                  /*!< grouped pin interrupt block clock */
    SYSCTL_CLOCK_DMA,                   /*!< DMA clock */
    SYSCTL_CLOCK_CRC,                   /*!< CRC clock */
    SYSCTL_CLOCK_WDT,                   /*!< WDT clock */
    SYSCTL_CLOCK_RTC,                   /*!< RTC clock */
    SYSCTL_CLOCK_ADC0 = 27,             /*!< ADC0 clock */
    SYSCTL_CLOCK_ADC1,                  /*!< ADC1 clock */
    SYSCTL_CLOCK_DAC,                   /*!< DAC clock */
    SYSCTL_CLOCK_ACMP,                  /*!< ACMP clock */
    /* Peripheral clock enables for SYSAHBCLKCTRL1 */
    SYSCTL_CLOCK_MRT = 32,              /*!< multi-rate timer clock */
    SYSCTL_CLOCK_RIT,                   /*!< repetitive interrupt timer clock */
    SYSCTL_CLOCK_SCT0,                  /*!< SCT0 clock */
    SYSCTL_CLOCK_SCT1,                  /*!< SCT1 clock */
    SYSCTL_CLOCK_SCT2,                  /*!< SCT2 clock */
    SYSCTL_CLOCK_SCT3,                  /*!< SCT3 clock */
    SYSCTL_CLOCK_SCTIPU,                /*!< SCTIPU clock */
    SYSCTL_CLOCK_CAN,                   /*!< CAN clock */
    SYSCTL_CLOCK_SPI0 = 32 + 9,         /*!< SPI0 clock */
    SYSCTL_CLOCK_SPI1,                  /*!< SPI1 clock */
    SYSCTL_CLOCK_I2C0 = 32 + 13,        /*!< I2C0 clock */
    SYSCTL_CLOCK_UART0 = 32 + 17,       /*!< UART0 clock */
    SYSCTL_CLOCK_UART1,                 /*!< UART1 clock */
    SYSCTL_CLOCK_UART2,                 /*!< UART2 clock */
    SYSCTL_CLOCK_QEI = 32 + 21,         /*!< QEI clock */
    SYSCTL_CLOCK_USB = 32 + 23,         /*!< USB clock */
} CHIP_SYSCTL_CLOCK_T;

/* De-assert reset for a peripheral */

void Chip_SYSCTL_AssertPeriphReset(CHIP_SYSCTL_PERIPH_RESET_T periph)
{
    if (periph >= 32) {
        LPC_SYSCON->PRESETCTRL1 |= (1 << ((uint32_t) periph - 32));
    }
    else {
        LPC_SYSCON->PRESETCTRL0 |= (1 << (uint32_t) periph);
    }   
}

/* Assert reset for a peripheral */

void Chip_SYSCTL_DeassertPeriphReset(CHIP_SYSCTL_PERIPH_RESET_T periph)
{
    if (periph >= 32) {
        LPC_SYSCON->PRESETCTRL1 &= ~(1 << ((uint32_t) periph - 32));
    }
    else {
        LPC_SYSCON->PRESETCTRL0 &= ~(1 << (uint32_t) periph);
    }
}


void Chip_SYSCTL_PeriphReset(CHIP_SYSCTL_PERIPH_RESET_T periph)
{
    Chip_SYSCTL_AssertPeriphReset(periph);
    Chip_SYSCTL_DeassertPeriphReset(periph);
}

/* Enable a system or peripheral clock */

void Chip_Clock_EnablePeriphClock(CHIP_SYSCTL_CLOCK_T clk)
{
    uint32_t clkEnab = (uint32_t) clk;

    if (clkEnab >= 32) {
        LPC_SYSCON->SYSAHBCLKCTRL1 |= (1 << (clkEnab - 32));
    }
    else {
        LPC_SYSCON->SYSAHBCLKCTRL0 |= (1 << clkEnab);
    }
}

void Chip_EEPROM_init()
{
    Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_EEPROM);
    Chip_SYSCTL_PeriphReset(RESET_EEPROM);
}

#endif